import {
  Dialog,
  DialogTitle,
  DialogContent,
  Typography,
  Card,
  Box,
  DialogActions,
  Button,
  TextField,
} from '@mui/material'
import { useEffect, useState } from 'react'
import { MaProduct, MaRelease } from 'solar-js-sdk'
import { unified } from 'unified'
import rehypeSanitize from 'rehype-sanitize'
import rehypeStringify from 'rehype-stringify'
import remarkGfm from 'remark-gfm'
import remarkParse from 'remark-parse'
import remarkRehype from 'remark-rehype'
import { InstallTask } from '@main/tasks'
import { useNavigate } from 'react-router'

export function MaInstallDialog({
  release,
  product,
  open,
  onClose,
}: {
  release?: MaRelease
  product?: MaProduct
  open: boolean
  onClose: () => void
}): JSX.Element {
  const [content, setContent] = useState<string>()

  const [installPath, setInstallPath] = useState<string>()

  async function parseContent(content: string): Promise<string> {
    content = content.replace(
      /!\[.*?\]\(solink:\/\/attachments\/([\w-]+)\)/g,
      '![alt](https://api.sn.solsynth.dev/cgi/uc/attachments/$1)',
    )

    const out = await unified()
      .use(remarkParse)
      .use(remarkRehype)
      .use(remarkGfm)
      .use(rehypeSanitize)
      .use(rehypeStringify)
      .process(content)

    return String(out)
  }

  useEffect(() => {
    if (release == null) return
    parseContent(release.meta.content).then((out) => setContent(out))
  }, [release])

  function checkPlatformAvailable(platform: string): boolean {
    return Object.keys(release?.assets ?? {}).includes(platform)
  }

  function selectInstallPath() {
    window.electron.ipcRenderer.invoke('show-path-select').then((out: string) => setInstallPath(out))
  }

  const navigate = useNavigate()

  function installApp() {
    const task: InstallTask = {
      release: release!,
      product: product!,
      basePath: installPath!,
    }
    window.electron.ipcRenderer.invoke('install-product-release', JSON.stringify(task))
    navigate('/tasks')
  }

  return (
    <Dialog open={open} onClose={onClose} maxWidth="sm" fullWidth>
      <DialogTitle>Install {release?.version}</DialogTitle>
      <DialogContent>
        <Typography variant="subtitle1" fontWeight="bold" component="h2">
          {release?.meta.title}
        </Typography>
        <Typography variant="body1" component="p" gutterBottom>
          {release?.meta.description}
        </Typography>

        <Card variant="outlined" sx={{ my: 2, maxHeight: '280px', overflowY: 'auto' }}>
          <Box
            component="article"
            className="prose prose-md dark:prose-invert"
            sx={{ p: 2 }}
            dangerouslySetInnerHTML={{ __html: content ?? '' }}
          />
        </Card>

        <Typography variant="subtitle1" fontWeight="bold" component="h2">
          Supported Platforms
        </Typography>
        <Typography variant="body1" component="p" gutterBottom fontFamily="monospace" fontSize={13}>
          {Object.keys(release?.assets ?? {}).join(', ')}
        </Typography>

        <TextField
          placeholder="Install Path"
          size="small"
          value={installPath}
          onClick={selectInstallPath}
          slotProps={{ htmlInput: { readOnly: true } }}
          sx={{ mt: 2 }}
          fullWidth
        />
      </DialogContent>
      <DialogActions>
        {!checkPlatformAvailable(window.platform) && (
          <Typography variant="caption" component="p" sx={{ flexGrow: 1, px: 2, opacity: 0.75 }}>
            This release is not supported on your current platform.
          </Typography>
        )}
        <Button onClick={onClose}>Cancel</Button>
        <Button autoFocus disabled={!checkPlatformAvailable(window.platform) || !installPath} onClick={installApp}>
          Install
        </Button>
      </DialogActions>
    </Dialog>
  )
}
